//
// Copyright (C) 2006 United States Government as represented by the
// Administrator of the National Aeronautics and Space Administration
// (NASA).  All Rights Reserved.
// 
// This software is distributed under the NASA Open Source Agreement
// (NOSA), version 1.3.  The NOSA has been approved by the Open Source
// Initiative.  See the file NOSA-1.3-JPF at the top of the distribution
// directory tree for the complete NOSA document.
// 
// THE SUBJECT SOFTWARE IS PROVIDED "AS IS" WITHOUT ANY WARRANTY OF ANY
// KIND, EITHER EXPRESSED, IMPLIED, OR STATUTORY, INCLUDING, BUT NOT
// LIMITED TO, ANY WARRANTY THAT THE SUBJECT SOFTWARE WILL CONFORM TO
// SPECIFICATIONS, ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
// A PARTICULAR PURPOSE, OR FREEDOM FROM INFRINGEMENT, ANY WARRANTY THAT
// THE SUBJECT SOFTWARE WILL BE ERROR FREE, OR ANY WARRANTY THAT
// DOCUMENTATION, IF PROVIDED, WILL CONFORM TO THE SUBJECT SOFTWARE.
//
package gov.nasa.jpf.symbc.perfploter;

import gov.nasa.jpf.jvm.JVM;
import gov.nasa.jpf.search.heuristic.HeuristicState;
import gov.nasa.jpf.symbc.numeric.PathCondition;

public class PerfPloterHeuristicState extends HeuristicState {
  
	private PathCondition pathCondition;
	private double pathProbability;
	private long pathDiversity;
	private long pathInstruction;
	
	private String branchIfInfo;
	private int nBreakOut;
    
	public PerfPloterHeuristicState (JVM vm, PathCondition pathCondition, double pathProbability, long pathDiversity, 
			long pathInstruction, String branchIfInfo, int nBreakOut) {
		super(vm);
		
		this.pathCondition = pathCondition;
		this.pathProbability = pathProbability;
		this.pathDiversity = pathDiversity;
		this.pathInstruction = pathInstruction;
		this.branchIfInfo = branchIfInfo;
		this.nBreakOut = nBreakOut;
	}

	public PathCondition getPathCondition() {
		return pathCondition;
	}

	public void setPathCondition(PathCondition pathCondition) {
		this.pathCondition = pathCondition;
	}

	public double getPathProbability() {
		return pathProbability;
	}

	public void setPathProbability(double pathProbability) {
		this.pathProbability = pathProbability;
	}

	public long getPathDiversity() {
		return pathDiversity;
	}

	public void setPathDiversity(long pathDiversity) {
		this.pathDiversity = pathDiversity;
	}

	public long getPathInstruction() {
		return pathInstruction;
	}

	public void setPathInstruction(long pathInstruction) {
		this.pathInstruction = pathInstruction;
	}

	public String getBranchIfInfo() {
		return branchIfInfo;
	}

	public void setBranchIfInfo(String branchIfInfo) {
		this.branchIfInfo = branchIfInfo;
	}

	public int getnBreakOut() {
		return nBreakOut;
	}

	public void setnBreakOut(int nBreakOut) {
		this.nBreakOut = nBreakOut;
	}

	@Override
	public String toString() {
		if (pathCondition != null) {
			return "PerfPloterState [stateId = " + stateId + ", pathCondition = " + pathCondition.header.stringPC()
					+ ", pathProbability = " + pathProbability + ", pathDiversity = " + pathDiversity
					+ ", pathInstruction = " + pathInstruction + ", branchIfInfo = " + branchIfInfo
					+ ", nBreakOut = " + nBreakOut + "]";
		} else {
			return null;
		}
	}

}
