package za.ac.sun.cs.solver;

import static org.junit.Assert.*;
import org.junit.Test;

import za.ac.sun.cs.solver.canonization.Canonizer;
import za.ac.sun.cs.solver.canonization.DefaultCanonizer;
import za.ac.sun.cs.solver.csp.CVC3;
import za.ac.sun.cs.solver.slicing.DefaultSlicer;
import za.ac.sun.cs.solver.slicing.Slicer;
import za.ac.sun.cs.solver.store.RedisStore;
import za.ac.sun.cs.solver.store.Store;

public class SolverTest {

	private Solver createSolver() {
		Solver solver = new Solver();
		assertFalse(solver.isSlicing());
		assertNull(solver.getSlicer());
		assertFalse(solver.isCanonizing());
		assertNull(solver.getCanonizer());
		assertNull(solver.getConstraintSolver());
		assertNull(solver.getDecisionProcedure());
		assertNull(solver.getModelCounter());
		assertFalse(solver.isStoring());
		assertNull(solver.getStore());
		return solver;
	}

	@Test
	public void testSolverConstruction01() {
		@SuppressWarnings("unused")
		Solver solver = createSolver();
	}

	@Test
	public void testSolverConstruction02() {
		Solver solver = createSolver();
		Slicer slicer = new DefaultSlicer();
		solver.setSlicer(slicer);
		assertTrue(solver.isSlicing());
		assertEquals(slicer, solver.getSlicer());
	}

	@Test
	public void testSolverConstruction03() {
		Solver solver = createSolver();
		Canonizer canonizer = new DefaultCanonizer();
		solver.setCanonizer(canonizer);
		assertTrue(solver.isCanonizing());
		assertEquals(canonizer, solver.getCanonizer());
	}

	@Test
	public void testSolverConstruction04() {
		Solver solver = createSolver();
		CVC3 cvc3 = new CVC3();
		solver.setConstraintSolver(cvc3);
		assertEquals(cvc3, solver.getConstraintSolver());
		assertNull(solver.getDecisionProcedure());
		assertNull(solver.getModelCounter());
	}

	@Test
	public void testSolverConstruction05() {
		Solver solver = createSolver();
		CVC3 cvc3 = new CVC3();
		solver.setDecisionProcedure(cvc3);
		assertNull(solver.getConstraintSolver());
		assertEquals(cvc3, solver.getDecisionProcedure());
		assertNull(solver.getModelCounter());
	}

	@Test
	public void testSolverConstruction07() {
		Solver solver = createSolver();
		Store store = new RedisStore();
		solver.setStore(store);
		assertTrue(solver.isStoring());
		assertEquals(store, solver.getStore());
	}

	@Test
	public void testSolverConstruction08() {
		Solver solver = createSolver();
		Slicer slicer = new DefaultSlicer();
		solver.setSlicer(slicer);
		Canonizer canonizer = new DefaultCanonizer();
		solver.setCanonizer(canonizer);
		CVC3 cvc3 = new CVC3();
		solver.setConstraintSolver(cvc3);
		solver.setDecisionProcedure(cvc3);
		Store store = new RedisStore();
		solver.setStore(store);
		assertTrue(solver.isSlicing());
		assertEquals(slicer, solver.getSlicer());
		assertTrue(solver.isCanonizing());
		assertEquals(canonizer, solver.getCanonizer());
		assertEquals(cvc3, solver.getConstraintSolver());
		assertEquals(cvc3, solver.getDecisionProcedure());
		assertNull(solver.getModelCounter());
		assertTrue(solver.isStoring());
		assertEquals(store, solver.getStore());
	}

}
