package za.ac.sun.cs.solver.canonization;

import static org.junit.Assert.*;

import java.util.Arrays;
import java.util.List;

import org.junit.Test;

import za.ac.sun.cs.solver.Instance;
import za.ac.sun.cs.solver.Solver;
import za.ac.sun.cs.solver.expr.Expression;
import za.ac.sun.cs.solver.expr.IntConstant;
import za.ac.sun.cs.solver.expr.IntVariable;
import za.ac.sun.cs.solver.expr.Operation;
import za.ac.sun.cs.solver.slicing.DefaultSlicer;
import za.ac.sun.cs.solver.slicing.Slicer;

public class DefaultCanonizerTest {

	private Solver createSolver() {
		Solver solver = new Solver();
		Slicer slicer = new DefaultSlicer();
		solver.setSlicer(slicer);
		Canonizer canonizer = new DefaultCanonizer();
		solver.setCanonizer(canonizer);
		return solver;
	}

	private void finalCheckSlicing(String sliced, String[] slicedStrs) {
		String s0 = sliced.replaceAll("[()]", "");
		String s1 = s0.replaceAll("v[0-9]", "v");
		List<String> s2 = Arrays.asList(s1.split("&&"));
		List<String> s3 = Arrays.asList(slicedStrs);
		assertEquals(s2.size(), s3.size());
		for (int i = 0; i < s2.size(); i++) {
			assertEquals(s3.get(i), s2.get(i));
		}
//		for (String s : slicedStrs) {
//			int p = sliced.indexOf(s);
//			assertTrue("String \"" + s + "\" not found in \"" + sliced + "\"", p >= 0);
//			if (p == 0) {
//				sliced = sliced.substring(p + s.length());
//			} else if (p > 0) {
//				sliced = sliced.substring(0, p - 1) + sliced.substring(p + s.length());
//			}
//		}
//		sliced = sliced.replaceAll("[()&]", "");
//		assertEquals("", sliced);
	}

	private void checkCanonization(Solver solver, Expression expression, String fullStr, String... canonizedStrs) {
		Instance i = new Instance(solver, null, expression);
		Expression e = i.getExpression();
		assertTrue(e.equals(expression));
		assertEquals(expression.toString(), e.toString());
		assertEquals(fullStr, i.getFullExpression().toString());
		finalCheckSlicing(i.getCanonizedExpression().toString(), canonizedStrs);
	}

	private void checkCanonization(Solver solver, Expression expression, Expression parentExpression, String fullStr, String... canonizedStrs) {
		Instance i1 = new Instance(solver, null, parentExpression);
		Instance i2 = new Instance(solver, i1, expression);
		Expression e = i2.getExpression();
		assertTrue(e.equals(expression));
		assertEquals(expression.toString(), e.toString());
		assertEquals(fullStr, i2.getFullExpression().toString());
		finalCheckSlicing(i2.getCanonizedExpression().toString(), canonizedStrs);
	}

	@Test
	public void testCanonization01() {
		Solver s = createSolver();
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(0);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkCanonization(s, o, "aa==0", "1*v==0");
	}

	@Test
	public void testCanonization02() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(0);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, c1);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntConstant c2 = new IntConstant(1);
		Operation o2 = new Operation(Operation.Operator.NE, v2, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCanonization(s, o3, "(aa==0)&&(bb!=1)", "1*v==0", "1*v+-1!=0");
	}

	@Test
	public void testCanonization03() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(0);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, c1);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntConstant c2 = new IntConstant(1);
		Operation o2 = new Operation(Operation.Operator.NE, v2, c2);
		checkCanonization(s, o1, o2, "(aa==0)&&(bb!=1)", "1*v==0");
	}

	@Test
	public void testCanonization04() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(0);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, c1);
		IntConstant c2 = new IntConstant(1);
		Operation o2 = new Operation(Operation.Operator.NE, v1, c2);
		checkCanonization(s, o1, o2, "(aa==0)&&(aa!=1)", "1*v==0", "1*v+-1!=0");
	}

	@Test
	public void testCanonization05() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, v2);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		Operation o2 = new Operation(Operation.Operator.EQ, v2, v3);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		Operation o3 = new Operation(Operation.Operator.EQ, v3, v4);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		Operation o4 = new Operation(Operation.Operator.EQ, v4, v5);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		Operation o234 = new Operation(Operation.Operator.AND, o2, o34);
		checkCanonization(s, o1, o234, "(aa==bb)&&((bb==cc)&&((cc==dd)&&(dd==ee)))", "1*v+-1*v==0", "1*v+-1*v==0", "1*v+-1*v==0", "1*v+-1*v==0");
	}

	@Test
	public void testCanonization06() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, v2);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		Operation o2 = new Operation(Operation.Operator.EQ, v2, v3);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		Operation o3 = new Operation(Operation.Operator.EQ, v3, v4);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		IntVariable v6 = new IntVariable("ff", 0, 99);
		Operation o4 = new Operation(Operation.Operator.EQ, v5, v6);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		Operation o234 = new Operation(Operation.Operator.AND, o2, o34);
		checkCanonization(s, o1, o234, "(aa==bb)&&((bb==cc)&&((cc==dd)&&(ee==ff)))", "1*v+-1*v==0", "1*v+-1*v==0", "1*v+-1*v==0");
	}

	@Test
	public void testCanonization07() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		IntVariable v6 = new IntVariable("ff", 0, 99);
		IntVariable v7 = new IntVariable("gg", 0, 99);
		Operation o1 = new Operation(Operation.Operator.LT, v1, new Operation(Operation.Operator.ADD, v2, v3));
		Operation o2 = new Operation(Operation.Operator.LT, v2, new Operation(Operation.Operator.ADD, v4, v5));
		Operation o3 = new Operation(Operation.Operator.LT, v3, new Operation(Operation.Operator.ADD, v6, v7));
		Operation o23 = new Operation(Operation.Operator.AND, o2, o3);
		checkCanonization(s, o1, o23, "(aa<(bb+cc))&&((bb<(dd+ee))&&(cc<(ff+gg)))", "1*v+-1*v+-1*v<0", "1*v+-1*v+-1*v<0", "1*v+-1*v+-1*v<0");
	}

	@Test
	public void testCanonization08() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		IntVariable v6 = new IntVariable("ff", 0, 99);
		IntVariable v7 = new IntVariable("gg", 0, 99);
		IntVariable v8 = new IntVariable("hh", 0, 99);
		Operation o1 = new Operation(Operation.Operator.LT, v1, new Operation(Operation.Operator.ADD, v2, v3));
		Operation o2 = new Operation(Operation.Operator.LT, v2, new Operation(Operation.Operator.ADD, v4, v5));
		Operation o3 = new Operation(Operation.Operator.LT, v6, new Operation(Operation.Operator.ADD, v7, v8));
		Operation o23 = new Operation(Operation.Operator.AND, o2, o3);
		checkCanonization(s, o1, o23, "(aa<(bb+cc))&&((bb<(dd+ee))&&(ff<(gg+hh)))", "1*v+-1*v+-1*v<0", "1*v+-1*v+-1*v<0");
	}

	@Test
	public void testCanonization09() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		Operation o1 = new Operation(Operation.Operator.ADD, v1, v2);
		Operation o2 = new Operation(Operation.Operator.ADD, v1, v3);
		Operation o3 = new Operation(Operation.Operator.LT, o1, o2);
		checkCanonization(s, o3, "(aa+bb)<(aa+cc)", "1*v+-1*v<0");
	}

	@Test
	public void testCanonization10() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		Operation o1 = new Operation(Operation.Operator.ADD, v1, v2);
		Operation o2 = new Operation(Operation.Operator.SUB, v1, v3);
		Operation o3 = new Operation(Operation.Operator.LT, o1, o2);
		checkCanonization(s, o3, "(aa+bb)<(aa-cc)", "1*v+1*v<0");
	}

	@Test
	public void testCanonization11() {
		Solver s = createSolver();
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.ADD, v1, v2);
		Operation o2 = new Operation(Operation.Operator.SUB, v2, v1);
		Operation o3 = new Operation(Operation.Operator.LT, o1, o2);
		checkCanonization(s, o3, "(aa+bb)<(bb-aa)", "2*v<0");
	}

	@Test
	public void testCanonization12() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntConstant c2 = new IntConstant(3);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.ADD, c1, c2);
		Operation o2 = new Operation(Operation.Operator.MUL, o1, v1);
		Operation o3 = new Operation(Operation.Operator.LT, o2, v2);
		checkCanonization(s, o3, "((2+3)*aa)<bb", "5*v+-1*v<0");
	}

	@Test
	public void testCanonization13() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntConstant c2 = new IntConstant(3);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.ADD, c1, c2);
		Operation o2 = new Operation(Operation.Operator.MUL, v1, o1);
		Operation o3 = new Operation(Operation.Operator.LT, o2, v2);
		checkCanonization(s, o3, "(aa*(2+3))<bb", "5*v+-1*v<0");
	}

	@Test
	public void testCanonization14() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.SUB, v1, v2);
		Operation o2 = new Operation(Operation.Operator.MUL, o1, c1);
		Operation o3 = new Operation(Operation.Operator.LT, o2, v1);
		checkCanonization(s, o3, "((aa-bb)*2)<aa", "1*v+-2*v<0");
	}

	@Test
	public void testCanonization15() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.SUB, v1, v2);
		Operation o2 = new Operation(Operation.Operator.MUL, c1, o1);
		Operation o3 = new Operation(Operation.Operator.LT, o2, v1);
		checkCanonization(s, o3, "(2*(aa-bb))<aa", "1*v+-2*v<0");
	}

	@Test
	public void testCanonization16() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntConstant c2 = new IntConstant(4);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		Operation o1 = new Operation(Operation.Operator.MUL, v1, c1);
		Operation o2 = new Operation(Operation.Operator.MUL, c2, v1);
		Operation o3 = new Operation(Operation.Operator.ADD, o1, o2);
		Operation o4 = new Operation(Operation.Operator.LT, o3, v2);
		checkCanonization(s, o4, "((aa*2)+(4*aa))<bb", "6*v+-1*v<0");
	}

	@Test
	public void testCanonization17() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		Operation o1 = new Operation(Operation.Operator.LT, c1, c1);
		checkCanonization(s, o1, "2<2", "0==1");
	}

	@Test
	public void testCanonization18() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		Operation o1 = new Operation(Operation.Operator.LT, c1, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v1, c1);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCanonization(s, o3, "(2<2)&&(aa<2)", "0==1");
	}

	@Test
	public void testCanonization19() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		Operation o1 = new Operation(Operation.Operator.LE, c1, c1);
		checkCanonization(s, o1, "2<=2", "0==0");
	}

	@Test
	public void testCanonization20() {
		Solver s = createSolver();
		IntConstant c1 = new IntConstant(2);
		IntVariable v1 = new IntVariable("aa", 0, 99);
		Operation o1 = new Operation(Operation.Operator.LE, c1, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v1, c1);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCanonization(s, o3, "(2<=2)&&(aa<2)", "1*v+-2<0");
	}

}
