package za.ac.sun.cs.solver.csp;

import static org.junit.Assert.*;
import org.junit.Test;

import za.ac.sun.cs.solver.Instance;
import za.ac.sun.cs.solver.Solver;
import za.ac.sun.cs.solver.canonization.DefaultCanonizer;
import za.ac.sun.cs.solver.expr.Expression;
import za.ac.sun.cs.solver.expr.IntConstant;
import za.ac.sun.cs.solver.expr.IntVariable;
import za.ac.sun.cs.solver.expr.Operation;
import za.ac.sun.cs.solver.slicing.DefaultSlicer;

public class CVC3Test1 {

	private Solver solver1 = null;

	private Solver solver2 = null;

	private Solver solver3 = null;

	private void createSolvers() {
		solver1 = new Solver();
		solver1.setDecisionProcedure(new CVC3());
		solver2 = new Solver();
		solver2.setSlicer(new DefaultSlicer());
		solver2.setDecisionProcedure(new CVC3());
		solver3 = new Solver();
		solver3.setSlicer(new DefaultSlicer());
		solver3.setCanonizer(new DefaultCanonizer());
		solver3.setDecisionProcedure(new CVC3());
	}

	private void checkSatisfiability(Expression expression, boolean result1, boolean result2, boolean result3) {
		createSolvers();
		Instance instance1 = new Instance(solver1, null, expression);
		assertEquals(result1, instance1.isSatisfiable());
		Instance instance2 = new Instance(solver2, null, expression);
		assertEquals(result2, instance2.isSatisfiable());
		Instance instance3 = new Instance(solver3, null, expression);
		assertEquals(result3, instance3.isSatisfiable());
	}

	private void checkSatisfiability(Expression expression, boolean result) {
		checkSatisfiability(expression, result, result, result);
	}

	private void checkSatisfiability(Expression expression, Expression parentExpression, boolean result1, boolean result2, boolean result3) {
		createSolvers();
		Instance instance1p = new Instance(solver1, null, parentExpression);
		Instance instance1 = new Instance(solver1, instance1p, expression);
		assertEquals(result1, instance1.isSatisfiable());
		Instance instance2p = new Instance(solver2, null, parentExpression);
		Instance instance2 = new Instance(solver2, instance2p, expression);
		assertEquals(result2, instance2.isSatisfiable());
		Instance instance3p = new Instance(solver3, null, parentExpression);
		Instance instance3 = new Instance(solver3, instance3p, expression);
		assertEquals(result3, instance3.isSatisfiable());
	}

	private void checkSatisfiability(Expression expression, Expression parentExpression, boolean result) {
		checkSatisfiability(expression, parentExpression, result, result, result);
	}

	@Test
	public void testCVC301() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(0);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkSatisfiability(o, true);
	}

	@Test
	public void testCVC302() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(100);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkSatisfiability(o, false);
	}

	@Test
	public void testCVC303() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(10);
		Operation o1 = new Operation(Operation.Operator.EQ, v, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSatisfiability(o3, true);
	}

	@Test
	public void testCVC304() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.EQ, v, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSatisfiability(o3, false);
	}

	@Test
	public void testCVC305() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSatisfiability(o3, true);
	}

	@Test
	public void testCVC306() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v, c2);
		checkSatisfiability(o1, o2, true);
	}

	@Test
	public void testCVC307() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(2012);
		Operation o1 = new Operation(Operation.Operator.GE, v1, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v2, c2);
		checkSatisfiability(o1, o2, false, true, true);
	}

	@Test
	public void testCVC308() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(2012);
		Operation o1 = new Operation(Operation.Operator.GE, v1, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v2, c2);
		checkSatisfiability(o2, o1, false);
	}

	@Test
	public void testCVC309() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		Operation o1 = new Operation(Operation.Operator.LT, v1, v2);
		Operation o2 = new Operation(Operation.Operator.LT, v2, v3);
		Operation o3 = new Operation(Operation.Operator.LT, v3, v4);
		Operation o4 = new Operation(Operation.Operator.LT, v4, v5);
		Operation o5 = new Operation(Operation.Operator.LT, v5, v1);
		Operation o45 = new Operation(Operation.Operator.AND, o4, o5);
		Operation o345 = new Operation(Operation.Operator.AND, o3, o45);
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		checkSatisfiability(o12, o345, false);
	}

	@Test
	public void testCVC310() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		Operation o1 = new Operation(Operation.Operator.LE, v1, v2);
		Operation o2 = new Operation(Operation.Operator.LE, v2, v3);
		Operation o3 = new Operation(Operation.Operator.LE, v3, v4);
		Operation o4 = new Operation(Operation.Operator.LE, v4, v5);
		Operation o5 = new Operation(Operation.Operator.LE, v5, v1);
		Operation o45 = new Operation(Operation.Operator.AND, o4, o5);
		Operation o345 = new Operation(Operation.Operator.AND, o3, o45);
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		checkSatisfiability(o12, o345, true);
	}

	@Test
	public void testCVC311() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntVariable v3 = new IntVariable("cc", 0, 99);
		IntVariable v4 = new IntVariable("dd", 0, 99);
		IntVariable v5 = new IntVariable("ee", 0, 99);
		IntConstant c1 = new IntConstant(2);
		Operation o1 = new Operation(Operation.Operator.EQ, v2, new Operation(Operation.Operator.MUL, c1, v1));
		Operation o2 = new Operation(Operation.Operator.EQ, v3, new Operation(Operation.Operator.MUL, c1, v2));
		Operation o3 = new Operation(Operation.Operator.EQ, v4, new Operation(Operation.Operator.MUL, c1, v3));
		Operation o4 = new Operation(Operation.Operator.EQ, v5, new Operation(Operation.Operator.MUL, c1, v4));
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		checkSatisfiability(o12, o34, true);
	}

	@Test
	public void testCVC312() {
		IntVariable v1 = new IntVariable("aa", 0, 9);
		IntVariable v2 = new IntVariable("bb", 0, 9);
		IntVariable v3 = new IntVariable("cc", 0, 9);
		IntVariable v4 = new IntVariable("dd", 0, 9);
		IntVariable v5 = new IntVariable("ee", 0, 9);
		IntConstant c1 = new IntConstant(2);
		Operation o1 = new Operation(Operation.Operator.EQ, v2, new Operation(Operation.Operator.MUL, c1, v1));
		Operation o2 = new Operation(Operation.Operator.EQ, v3, new Operation(Operation.Operator.MUL, c1, v2));
		Operation o3 = new Operation(Operation.Operator.EQ, v4, new Operation(Operation.Operator.MUL, c1, v3));
		Operation o4 = new Operation(Operation.Operator.EQ, v5, new Operation(Operation.Operator.MUL, c1, v4));
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		checkSatisfiability(o12, o34, true);
	}

	@Test
	public void testCVC313() {
		IntVariable v1 = new IntVariable("aa", 1, 9);
		IntVariable v2 = new IntVariable("bb", 0, 9);
		IntVariable v3 = new IntVariable("cc", 0, 9);
		IntVariable v4 = new IntVariable("dd", 0, 9);
		IntVariable v5 = new IntVariable("ee", 0, 9);
		IntConstant c1 = new IntConstant(2);
		Operation o1 = new Operation(Operation.Operator.EQ, v2, new Operation(Operation.Operator.MUL, c1, v1));
		Operation o2 = new Operation(Operation.Operator.EQ, v3, new Operation(Operation.Operator.MUL, c1, v2));
		Operation o3 = new Operation(Operation.Operator.EQ, v4, new Operation(Operation.Operator.MUL, c1, v3));
		Operation o4 = new Operation(Operation.Operator.EQ, v5, new Operation(Operation.Operator.MUL, c1, v4));
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		checkSatisfiability(o12, o34, false);
	}

	@Test
	public void testCVC314() {
		IntVariable v1 = new IntVariable("i1", 0, 1073741824);
		IntVariable v2 = new IntVariable("j2", 0, 1073741824);
		IntVariable v3 = new IntVariable("k3", 0, 1073741824);
		IntConstant c1 = new IntConstant(0);
		Operation o1 = new Operation(Operation.Operator.NE, v2, v3);
		Operation o2 = new Operation(Operation.Operator.EQ, v1, v3);
		Operation o3 = new Operation(Operation.Operator.EQ, v1, v2);
		Operation o4 = new Operation(Operation.Operator.GT, v1, c1);
		Operation o5 = new Operation(Operation.Operator.GT, v2, c1);
		Operation o6 = new Operation(Operation.Operator.GT, v3, c1);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		Operation o56 = new Operation(Operation.Operator.AND, o5, o6);
		Operation o234 = new Operation(Operation.Operator.AND, o2, o34);
		Operation o23456 = new Operation(Operation.Operator.AND, o234, o56);
		checkSatisfiability(o1, o23456, false);
	}

}
