package za.ac.sun.cs.solver.csp;

import static org.junit.Assert.assertEquals;

import org.junit.Test;

import za.ac.sun.cs.solver.Instance;
import za.ac.sun.cs.solver.Solver;
import za.ac.sun.cs.solver.canonization.DefaultCanonizer;
import za.ac.sun.cs.solver.expr.Expression;
import za.ac.sun.cs.solver.expr.IntConstant;
import za.ac.sun.cs.solver.expr.IntVariable;
import za.ac.sun.cs.solver.expr.Operation;
import za.ac.sun.cs.solver.slicing.DefaultSlicer;

public class CVC3Test2 {

	private Solver solver1 = null;

	private Solver solver2 = null;

	private Solver solver3 = null;

	private void createSolvers() {
		solver1 = new Solver();
		CVC3 cvc31 = new CVC3();
		solver1.setDecisionProcedure(cvc31);
		solver1.setConstraintSolver(cvc31);
		solver2 = new Solver();
		solver2.setSlicer(new DefaultSlicer());
		CVC3 cvc32 = new CVC3();
		solver2.setDecisionProcedure(cvc32);
		solver2.setConstraintSolver(cvc32);
		solver3 = new Solver();
		solver3.setSlicer(new DefaultSlicer());
		solver3.setCanonizer(new DefaultCanonizer());
		CVC3 cvc33 = new CVC3();
		solver3.setDecisionProcedure(cvc33);
		solver3.setConstraintSolver(cvc33);
	}

	private void checkSolution(Expression expression, IntVariable[] variables,
			Number[] results1, Number[] results2, Number[] results3) {
		int n = variables.length;
		assert n == results1.length;
		assert n == results2.length;
		assert n == results3.length;
		createSolvers();
		Instance instance1 = new Instance(solver1, null, expression);
		Instance instance2 = new Instance(solver2, null, expression);
		Instance instance3 = new Instance(solver3, null, expression);
		for (int i = 0; i < n; i++) {
			assertEquals(results1[i], instance1.getIntValue(variables[i]));
			assertEquals(results2[i], instance2.getIntValue(variables[i]));
			assertEquals(results3[i], instance3.getIntValue(variables[i]));
		}
	}

	private void checkSolution(Expression expression, IntVariable[] variables,
			Number[] results) {
		checkSolution(expression, variables, results, results, results);
	}

	private void checkSolution(Expression expression,
			Expression parentExpression, IntVariable[] variables,
			Number[] results1, Number[] results2, Number[] results3) {
		int n = variables.length;
		assert n == results1.length;
		assert n == results2.length;
		assert n == results3.length;
		createSolvers();
		Instance instance1p = new Instance(solver1, null, parentExpression);
		Instance instance1 = new Instance(solver1, instance1p, expression);
		Instance instance2p = new Instance(solver2, null, parentExpression);
		Instance instance2 = new Instance(solver2, instance2p, expression);
		Instance instance3p = new Instance(solver3, null, parentExpression);
		Instance instance3 = new Instance(solver3, instance3p, expression);
		for (int i = 0; i < n; i++) {
			assertEquals(results1[i], instance1.getIntValue(variables[i]));
			assertEquals(results2[i], instance2.getIntValue(variables[i]));
			assertEquals(results3[i], instance3.getIntValue(variables[i]));
		}
	}

	private void checkSolution(Expression expression,
			Expression parentExpression, IntVariable[] variables,
			Number[] results) {
		checkSolution(expression, parentExpression, variables, results,
				results, results);
	}

	@Test
	public void testCVC301() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(0);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkSolution(o, new IntVariable[] { v },
				new Integer[] { c.getValue() });
	}

	@Test
	public void testCVC302() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(100);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkSolution(o, new IntVariable[] { v }, new Integer[] { null });
	}

	@Test
	public void testCVC303() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(10);
		Operation o1 = new Operation(Operation.Operator.EQ, v, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSolution(o3, new IntVariable[] { v },
				new Integer[] { c1.getValue() });
	}

	@Test
	public void testCVC304() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.EQ, v, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSolution(o3, new IntVariable[] { v }, new Integer[] { null });
	}

	@Test
	public void testCVC305() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(11);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSolution(o3, new IntVariable[] { v },
				new Integer[] { c1.getValue() });
	}

	@Test
	public void testCVC306() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v, c2);
		checkSolution(o1, o2, new IntVariable[] { v },
				new Integer[] { c1.getValue() });
	}

	@Test
	public void testCVC307() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v2, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkSolution(o3, new IntVariable[] { v1, v2 },
				new Integer[] { c1.getValue(), c2.getValue() });
	}

	@Test
	public void testCVC308() {
		IntVariable v1 = new IntVariable("aa", 0, 99);
		IntVariable v2 = new IntVariable("bb", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.EQ, v1, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, v2, c2);
		checkSolution(o1, o2, new IntVariable[] { v1, v2 },
				new Integer[] { c1.getValue(), c2.getValue() }, new Integer[] {
						c1.getValue(), null }, new Integer[] { c1.getValue(),
						null });
	}

}
