package za.ac.sun.cs.solver.csp;

import static org.junit.Assert.*;

import org.apfloat.Apint;
import org.junit.Test;

import za.ac.sun.cs.solver.Instance;
import za.ac.sun.cs.solver.Solver;
import za.ac.sun.cs.solver.canonization.DefaultCanonizer;
import za.ac.sun.cs.solver.expr.Expression;
import za.ac.sun.cs.solver.expr.IntConstant;
import za.ac.sun.cs.solver.expr.IntVariable;
import za.ac.sun.cs.solver.expr.Operation;
import za.ac.sun.cs.solver.slicing.DefaultSlicer;

public class LattETest1 {

	private static final String LATTE_EXECUTABLE = "/home/jaco/latte-integrale-1.5.3/latte-int-1.5/code/latte/count";

	private Solver solver1 = null;

	private Solver solver2 = null;

	private void createSolvers() {
		solver1 = new Solver();
		solver1.setCanonizer(new DefaultCanonizer());
		LattE latte1 = new LattE(null);
		latte1.setLatteExecutable(LATTE_EXECUTABLE);
		solver1.setModelCounter(latte1);

		solver2 = new Solver();
		solver2.setSlicer(new DefaultSlicer());
		solver2.setCanonizer(new DefaultCanonizer());
		LattE latte2 = new LattE(null);
		latte2.setLatteExecutable(LATTE_EXECUTABLE);
		solver2.setModelCounter(latte2);
	}

	private void checkCount(Expression expression, int result1, int result2) {
		createSolvers();
		Instance instance1 = new Instance(solver1, null, expression);
		assertEquals(new Apint(result1), instance1.countSolutions());
		Instance instance2 = new Instance(solver2, null, expression);
		assertTrue(new Apint(result2).equals(instance2.countSolutions()));
	}

	private void checkCount(Expression expression, int result) {
		checkCount(expression, result, result);
	}

	@Test
	public void testLattE01() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(0);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkCount(o, 1);
	}

	@Test
	public void testLattE02() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(0);
		Operation o = new Operation(Operation.Operator.NE, v, c);
		checkCount(o, 99);
	}

	@Test
	public void testLattE03() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(10);
		Operation o = new Operation(Operation.Operator.EQ, v, c);
		checkCount(o, 1);
	}

	@Test
	public void testLattE04() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(10);
		Operation o = new Operation(Operation.Operator.LT, v, c);
		checkCount(o, 10);
	}

	@Test
	public void testLattE05() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(12);
		Operation o = new Operation(Operation.Operator.LE, v, c);
		checkCount(o, 13);
	}

	@Test
	public void testLattE06() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(90);
		Operation o = new Operation(Operation.Operator.GT, v, c);
		checkCount(o, 9);
	}

	@Test
	public void testLattE07() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c = new IntConstant(95);
		Operation o = new Operation(Operation.Operator.GE, v, c);
		checkCount(o, 5);
	}

	@Test
	public void testLattE08() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GT, v, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCount(o3, 9);
	}

	@Test
	public void testLattE09() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LT, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCount(o3, 10);
	}

	@Test
	public void testLattE10() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GT, v, c1);
		Operation o2 = new Operation(Operation.Operator.LE, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCount(o3, 10);
	}

	@Test
	public void testLattE11() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LE, v, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCount(o3, 11);
	}

	@Test
	public void testLattE12() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		IntConstant c3 = new IntConstant(15);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LE, v, c2);
		Operation o3 = new Operation(Operation.Operator.NE, v, c3);
		Operation o23 = new Operation(Operation.Operator.AND, o2, o3);
		Operation o123 = new Operation(Operation.Operator.AND, o1, o23);
		checkCount(o123, 10);
	}

	@Test
	public void testLattE13() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		IntConstant c3 = new IntConstant(15);
		IntConstant c4 = new IntConstant(13);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LE, v, c2);
		Operation o3 = new Operation(Operation.Operator.NE, v, c3);
		Operation o4 = new Operation(Operation.Operator.NE, v, c4);
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		Operation o1234 = new Operation(Operation.Operator.AND, o12, o34);
		checkCount(o1234, 9);
	}

	@Test
	public void testLattE14() {
		IntVariable v = new IntVariable("aa", 0, 99);
		IntConstant c1 = new IntConstant(10);
		IntConstant c2 = new IntConstant(20);
		IntConstant c3 = new IntConstant(15);
		IntConstant c4 = new IntConstant(13);
		IntConstant c5 = new IntConstant(17);
		Operation o1 = new Operation(Operation.Operator.GE, v, c1);
		Operation o2 = new Operation(Operation.Operator.LE, v, c2);
		Operation o3 = new Operation(Operation.Operator.NE, v, c3);
		Operation o4 = new Operation(Operation.Operator.NE, v, c4);
		Operation o5 = new Operation(Operation.Operator.NE, v, c5);
		Operation o12 = new Operation(Operation.Operator.AND, o1, o2);
		Operation o34 = new Operation(Operation.Operator.AND, o3, o4);
		Operation o1234 = new Operation(Operation.Operator.AND, o12, o34);
		Operation o12345 = new Operation(Operation.Operator.AND, o1234, o5);
		checkCount(o12345, 8);
	}

	@Test
	public void testLattE15() {
		IntVariable v = new IntVariable("aa", 0, 19);
		IntConstant c1 = new IntConstant(2);
		IntConstant c2 = new IntConstant(3);
		Operation o1 = new Operation(Operation.Operator.MUL, v, c1);
		Operation o2 = new Operation(Operation.Operator.GT, o1, c2);
		checkCount(o2, 18);
	}

	@Test
	public void testLattE16() {
		IntVariable v = new IntVariable("aa", 0, 19);
		IntConstant c1 = new IntConstant(3);
		IntConstant c2 = new IntConstant(15);
		Operation o1 = new Operation(Operation.Operator.MUL, v, c1);
		Operation o2 = new Operation(Operation.Operator.GT, o1, c2);
		checkCount(o2, 14);
	}

	@Test
	public void testLattE17() {
		IntVariable v = new IntVariable("aa", 0, 19);
		IntConstant c1 = new IntConstant(3);
		IntConstant c2 = new IntConstant(15);
		Operation o1 = new Operation(Operation.Operator.MUL, v, c1);
		Operation o2 = new Operation(Operation.Operator.EQ, o1, c2);
		checkCount(o2, 1);
	}

	@Test
	public void testLattE18() {
		IntVariable v = new IntVariable("aa", 0, 19);
		IntConstant c1 = new IntConstant(3);
		IntConstant c2 = new IntConstant(15);
		Operation o1 = new Operation(Operation.Operator.MUL, v, c1);
		Operation o2 = new Operation(Operation.Operator.NE, o1, c2);
		checkCount(o2, 19);
	}

	@Test
	public void testLattE19() {
		IntVariable v1 = new IntVariable("aa", 0, 9);
		IntVariable v2 = new IntVariable("bb", 0, 9);
		Operation o = new Operation(Operation.Operator.NE, v1, v2);
		checkCount(o, 90);
	}

	@Test
	public void testLattE20() {
		IntVariable v1 = new IntVariable("aa", 0, 9);
		IntVariable v2 = new IntVariable("bb", 0, 9);
		Operation o = new Operation(Operation.Operator.LT, v1, v2);
		checkCount(o, 45);
	}

	@Test
	public void testLattE21() {
		IntVariable v1 = new IntVariable("aa", 0, 9);
		IntVariable v2 = new IntVariable("bb", 0, 9);
		IntConstant c1 = new IntConstant(3);
		IntConstant c2 = new IntConstant(5);
		Operation o1 = new Operation(Operation.Operator.NE, v1, c1);
		Operation o2 = new Operation(Operation.Operator.NE, v2, c2);
		Operation o3 = new Operation(Operation.Operator.AND, o1, o2);
		checkCount(o3, 81);
	}

}
