package za.ac.sun.cs.solver.slicing;

import java.util.List;

import za.ac.sun.cs.solver.expr.Expression;

/**
 * An interface to a slicer. The purpose of a slicer is to remove the
 * unnecessary parts of an expression. For example, given the satisfiability of
 * <code>x&lt;y &amp;&amp; z<10</code> is exactly the same as the satisfiability
 * of <code>x&lt;y</code> when it is assumed that <code>z<10</code> is
 * satisfiable. In other words, given the "fresh" part of a path condition, and
 * the "non-fresh" part, the job of the slicer is to make the expression as
 * small as possible.
 * 
 * @author Jaco Geldenhuys <jaco@cs.sun.ac.za>
 */
public interface Slicer {

	/**
	 * Slices the expression (given in two parts) as much as possible.
	 * 
	 * @param fresh the fresh part of the path condition
	 * @param rest the non-fresh part of the path condition
	 * @return a reduced expression
	 */
	public Expression slice(Expression fresh, Expression rest);

	/**
	 * Returns a list of "report" lines about the performance of the slicer.
	 * 
	 * @return lines of a performance report
	 */
	public List<String> getSlicerReport();

}
